

#include "Arduino.h"            //import necessary libraries
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"

#include <Servo.h>

#define servo3pin 2   //servo pins. 3 is in the 3 o'clock position and so on
#define servo6pin 3   //
#define servo9pin 4   //
#define servo12pin 5  //

#define photo A0   //photoresistor read pin

#define DFRX 10    //RX pin for the DFPlayer
#define DFTX 11    //TX pin for the DFPlayer

#define dfvol 25  //DFPlayer volume setting

#define mutepin  7 //pins for muting and freezing the machine
#define stillpin 8 //

#define advancepin 6 //sets pin for manually advancing the hour on the clock for setting and testing purposes

#define threshold 800 //threshold for photoresistor activation

int pvalue;       //itialize needed variables
int noisecounter = 0;
int currTime = 0;
int toplay;
bool isLast0 = false;
bool advance = false;

Servo servo3;    //declare servo objects
Servo servo6;
Servo servo9;
Servo servo12;

SoftwareSerial DFSerial(DFRX, DFTX);  //initializes the software serial and DFplayer objects
DFRobotDFPlayerMini player;           //

void setup() {
  pinMode(photo, INPUT);              //set input pins to input mode
  pinMode(mutepin, INPUT_PULLUP);     //
  pinMode(stillpin, INPUT_PULLUP);    //
  pinMode(advancepin, INPUT_PULLUP);  //

  servo3.write(0);                    //write all servos to zero first. This means no twitching when they first attach (unless they're not at zero already).
  servo6.write(90);                    //
  servo9.write(0);                    //
  servo12.write(0);                   //
  servo3.attach(servo3pin);           //attach each servo so they go to their initial positions
  servo6.attach(servo6pin);           //
  servo9.attach(servo9pin);           //
  servo12.attach(servo12pin);         //
  delay(100);                         //give it a bit of delay to move
  servo3.detach();                    //Detach each servo. It is necessary to leave each servo detached until they are moved because of nuance
  servo6.detach();                    //in the way that the servo library and the DFPlayer works (basically, the servo library requires timers, and so does the DFPlayer, and when the DFPlayer takes over the timers the servos twitch)
  servo9.detach();                    //
  servo12.detach();                   //

  DFSerial.begin(9600);     //begin serial channels
  Serial.begin(115200);     //

  if(!player.begin(DFSerial)){    //make sure the DFPlayer initializes properly
    Serial.println("DFPlayer serial setup failed. Restart Manually.");
    while(true);
  }

  delay(1500);

  player.volume(dfvol);   //set DFPlayer volume

  Serial.println("setup complete");
}

void loop() {
  evaluate(); //update system state
  
  if (digitalRead(mutepin) && advance) player.play(toplay);  //run the sound and motion routines if the respective switches are in the right position and it is time to do so
  if (digitalRead(stillpin) && advance) movement();
  if(advance) advance = false;   //if we advanced, make sure to reset that flag
  delay(150);                    //limit the loop speed
}


void evaluate() {   //this function updates the system state
  
  pvalue = analogRead(photo);  //read photoresistor value and print to debug serial
  Serial.print("Photoresistor value: ");
  Serial.println(pvalue);

  if (pvalue <= threshold) noisecounter ++;  //if it's below the threshold, the minute hand is likely in the way. once this has happened three times we can be sure about it.
  else {
    noisecounter = 0;
    isLast0 = false;
  }

  if ((noisecounter > 3 &&  !isLast0) || !digitalRead(advancepin)){   //once we've counted three rounds of darkness or the advance button is pushed, the detector is triggered and it is time to advance
    Serial.println("Detector triggered");
    advance = true;
    
    currTime ++;        //increment the current time
    toplay = currTime;  //set the track to be played
    currTime = currTime % 12;  //make sure the current time is 0-11
    isLast0 = true;   //flag keeps the detector from triggering multiple times in one pass
  }
 
 }


void movement() {
  //move the correct servo
  // Each section follows the same format
  Serial.println(toplay);
  switch (toplay){
    case 3:
      servo3.attach(servo3pin);     //attach the servo to get it ready
      for (int i = 0; i < 35; i++){  //slowly sweep the servo through the angles between the start and end
        servo3.write(i);
        delay(100);
      }
      delay(250);
      servo3.write(0);  //reset servo to initial position
      delay(500);  //give it time to get there
      servo3.detach();  //detach it so it doesn't twitch.
      break;
      
    case 6: 
      servo6.attach(servo6pin);
      for (int i = 0; i < 31; i++){
        servo6.write(90-2*i);
        delay(50);
      }
      delay(750);
      servo6.write(90);
      delay(500);
      servo6.detach();
      break;
      
    case 9:
      servo9.attach(servo9pin);
      for (int i = 0; i < 25; i++){
        servo9.write(i);
        delay(100);
      }
      servo9.write(0);
      delay(500);
      servo9.detach();
    
      break;
      
    case 12:
      servo12.attach(servo12pin);
      for (int i = 0; i < 31; i++){
        servo12.write(i*3);
        delay(5);
      }
      delay(1000);
      servo12.write(0);
      delay(500);
      servo12.detach();
    
      break;
      
    default:
      break;
  }
}
